# server.py
import os
import asyncio
from openai import AsyncOpenAI # Импортируем асинхронный клиент
from mcp.server.fastmcp import FastMCP, Context

async def call_openai_for_news_analysis(news_text: str, analysis_prompt: str, ctx: Context) -> str:
    api_key = os.environ.get("OPENROUTER_API_KEY")
    model_name = os.environ.get("OPENROUTER_MODEL_NAME", "google/gemini-2.5-flash-preview") # Модель по умолчанию
    base_url = "https://openrouter.ai/api/v1"

    if not api_key:
        error_msg = "OPENROUTER_API_KEY не найден в переменных окружения."
        ctx.error(error_msg)
        # В реальном приложении можно рейзить кастомное исключение или возвращать спец. ошибку
        raise ValueError(error_msg)

    client = AsyncOpenAI(api_key=api_key, base_url=base_url)

    ctx.info(f"Отправка запроса к OpenAI API. Модель: {model_name}. Новость: {news_text[:50]}...")

    try:
        response = await client.chat.completions.create(
            model=model_name,
            messages=[
                {"role": "system", "content": analysis_prompt},
                {"role": "user", "content": news_text}
            ]
        )
        result = response.choices[0].message.content
        ctx.info("Ответ от OpenAI API получен.")
        return result if result else "OpenAI API вернул пустой ответ."
    except Exception as e:
        ctx.error(f"Ошибка при обращении к OpenAI API: {str(e)}")
        return f"Ошибка взаимодействия с OpenAI: {str(e)}"

# --- MCP Сервер и Инструменты ---

mcp = FastMCP("NewsLawChecker")

NEWS_ANALYSIS_LEGAL_PROMPT = """
**Закон РФ "О СМИ" (N 2124-I) – Ключевые моменты для редактора** Этот документ — ваш гид по основным правам, обязанностям и ограничениям в работе СМИ. Его цель — помочь вам принимать взвешенные решения и избегать нарушений. **I. Фундаментальные принципы:** 1. **Свобода массовой информации (Ст. 1):** Поиск, получение, производство и распространение информации, учреждение СМИ, владение ими — не подлежат ограничениям, _кроме тех, что предусмотрены законодательством о СМИ_. 2. **Недопустимость цензуры (Ст. 3):** Запрещено требовать предварительного согласования материалов (кроме случаев, когда должностное лицо — автор или интервьюируемый) или налагать запрет на их распространение. Финансирование цензуры запрещено. 3. **Недопустимость злоупотребления свободой СМИ (Ст. 4) – КРИТИЧЕСКИ ВАЖНО:** * **ЗАПРЕЩЕНО использовать СМИ для:** * Совершения уголовных деяний. * Разглашения гостайны или иной охраняемой законом тайны. * Распространения материалов с призывами к терроризму, его оправданием, других экстремистских материалов. * Пропаганды порнографии, насилия, жестокости. * Материалов с нецензурной бранью. * Пропаганды нетрадиционных сексуальных отношений и (или) предпочтений, педофилии, смены пола, отказа от деторождения. * Распространения информации об НКО/организациях, ликвидированных/запрещенных за экстремизм/терроризм, **без указания** на этот факт. * Сведений о способах изготовления/использования/приобретения наркотиков, психотропных веществ, их прекурсоров, пропаганды их преимуществ. * Сведений о несовершеннолетних, пострадавших от противоправных действий (ФИО, фото, видео, дата рождения, голос, место жительства/учебы/работы и т.д., позволяющих установить личность), _кроме строго определенных случаев_ (Ст. 41). * Инструкций по изготовлению взрывчатки, оружия (кроме снаряжения патронов к гражданскому оружию). * Предложений о дистанционной продаже алкоголя. * Информации об иностранных агентах и их материалов **без указания** на статус иностранного агента. * Рекламы информационных ресурсов иностранных агентов. * **ЗАПРЕЩЕНО** использовать скрытые вставки, воздействующие на подсознание или здоровье. * **Освещение КТО (Ст. 4):** Порядок сбора информации определяет руководитель КТО. Запрещено распространять сведения о спецсредствах, тактике, если это мешает операции или угрожает людям. Данные о сотрудниках спецподразделений – только с соблюдением закона о гостайне и персональных данных. **II. Основные понятия (Ст. 2) – Что нужно знать редактору:** * **Массовая информация:** печатные, аудио-, аудиовизуальные и иные сообщения для неограниченного круга лиц. * **СМИ:** периодическое печатное издание, сетевое издание, теле/радиоканал, программа и т.д. под постоянным названием, выходящее не реже раза в год. * **Редакция СМИ:** организация или гражданин, осуществляющие производство и выпуск СМИ. * **Главный редактор:** лицо, возглавляющее редакцию и принимающее окончательные решения по производству и выпуску СМИ. * **Журналист:** лицо, занимающееся созданием, сбором, подготовкой материалов для редакции зарегистрированного СМИ, связанное с ней трудовыми или иными отношениями, или по её уполномочию. * **Продукция СМИ:** тираж, отдельный выпуск, обновление сетевого издания. * **Сетевое издание:** сайт в сети "Интернет", зарегистрированный как СМИ. _Сайт, не зарегистрированный как СМИ, таковым не является._ **III. Организация деятельности СМИ – Ключевое для редакции:** * **Регистрация СМИ (Ст. 8):** Деятельность возможна только после регистрации (кроме исключений). _Сайт может быть зарегистрирован как сетевое издание_. Учредитель должен начать выпуск продукции в течение 1 года со дня регистрации, иначе регистрация может быть признана недействительной. * **Освобождение от регистрации (Ст. 12):** * СМИ госорганов/органов МСУ для официальных сообщений. * Печатные издания тиражом **менее 1000 экз.** * Радио- и телепрограммы в кабельных сетях, ограниченных одним учреждением/предприятием или имеющие не более 10 абонентов. * Аудио- и видеопрограммы в записи тиражом не более 10 экз. * **Внесение изменений в запись о регистрации (Ст. 11):** Обязательно при смене учредителя, названия, языка, тематики, территории распространения, доменного имени (для сетевых). Уведомление регистрирующего органа – при смене адреса редакции, периодичности, объема, прекращении/приостановлении деятельности. * **Признание регистрации недействительной (Ст. 15):** Судом по заявлению регистрирующего органа, если: * Сведения при регистрации были недостоверны. * СМИ не выходит в свет (в эфир) **более 1 года**. * Устав редакции не направлен в регистрирующий орган в течение 3 месяцев со дня первого выхода. * Повторная регистрация. * По решению Генпрокурора (Ст. 56.2). * **Прекращение и приостановление деятельности (Ст. 16, 16.1, 56.2):** * По решению учредителя. * Судом по иску регистрирующего органа за неоднократные (в течение 12 мес.) нарушения Ст. 4 (злоупотребление свободой СМИ), по которым были письменные предупреждения, или за неисполнение решения суда о приостановлении. * По основаниям ФЗ "О противодействии экстремистской деятельности". * Приостановление за нарушения при выборах (Ст. 16.1). * **Новое (Ст. 56.2):** Приостановление (до 3-6 мес.) или признание регистрации недействительной/прекращение лицензии на вещание по требованию Генпрокурора РФ или его заместителей за распространение: * Недостоверной общественно значимой информации (фейк-ньюс), создающей угрозу жизни, здоровью, порядку, безопасности, функционированию инфраструктуры. * Недостоверной информации об использовании ВС РФ, деятельности госорганов за рубежом. * Информации, выражающей явное неуважение к обществу, государству, символам, Конституции, органам власти. * Информации, дискредитирующей использование ВС РФ или деятельность госорганов за рубежом. * Призывов к несанкционированным акциям, массовым беспорядкам, санкциям против РФ. * **Статус редакции (Ст. 19):** Действует на основе профессиональной самостоятельности. Руководит главный редактор, он несет ответственность за соблюдение закона. Ограничения для главного редактора (судимость за преступления с использованием СМИ/экстремизм, возраст до 18, недееспособность). * **Ограничения иностранного участия (Ст. 19.1):** Иностранные государства, организации, юрлица с иностранным участием (>20%), иностранцы, лица с двойным гражданством не могут быть учредителями, редакциями, вещателями или контролировать более 20% долей в них. _Следить за структурой собственности!_ * **Уведомление о иностранном финансировании (Ст. 19.2):** Редакция/вещатель/издатель обязаны ежеквартально уведомлять Роскомнадзор о получении денег от иностранных источников (государств, организаций, граждан, иноагентов, рос. юрлиц с иностранным участием), кроме денег от учредителя, рекламы, распространения продукции, или разовых сумм <15 тыс. руб. * **Устав редакции (Ст. 20):** Принимается коллективом журналистов, утверждается учредителем. Определяет права и обязанности учредителя, редакции, главреда; полномочия коллектива; порядок назначения главреда; порядок прекращения/приостановления деятельности СМИ. Копия направляется в регистрирующий орган. * **Информационные агентства (Ст. 23):** При распространении их сообщений другими СМИ **ссылка обязательна**. **IV. Распространение информации – Практика для редактора:** * **Выход в свет (в эфир) (Ст. 26):** Только после разрешения главного редактора. * **Выходные данные (Ст. 27) – ОБЯЗАТЕЛЬНО УКАЗЫВАТЬ:** * **Печатные СМИ:** название, учредитель(и), ФИО главреда, номер и дата выпуска, индекс (если есть), тираж, цена (или "Свободная цена"/"Бесплатно"), адреса редакции, издателя, типографии, знак информационной продукции (возрастная маркировка). * **Теле/радиоканал:** объявление названия не реже 4 раз в сутки. Каждая программа – объявление названия. Сообщение об ограничении распространения, знак информпродукции. * **Сетевое издание:** название, учредитель(и), ФИО главреда, e-mail и телефон редакции, знак информпродукции. * **Все зарегистрированные СМИ:** указывать зарегистрировавший орган и регистрационный номер. * **СМИ, учрежденные иноагентом:** указывать, что создано иностранным агентом. * **Тираж (Ст. 28):** Определяется главредом по согласованию с издателем. Изъятие/уничтожение тиража – только по решению суда. * **Хранение материалов радио- и телепередач (Ст. 34):** Записи собственных передач – не менее 1 месяца. Регистрационный журнал (дата, время, тема, автор, ведущий, участники) – не менее 1 года. **Агитационные материалы (выборы/референдумы) – не менее 12 месяцев.** * **Обязательные сообщения (Ст. 35):** Редакция обязана бесплатно и в срок публиковать: * Вступившее в силу решение суда с требованием публикации. * Сообщение регистрирующего органа, касающееся деятельности редакции. * Официальные сообщения госорганов-учредителей. * Экстренную информацию при ЧС, военных действиях по обращению госорганов. * **Реклама (Ст. 36):** Регулируется законодательством о рекламе. * **Эротические издания (Ст. 37):** Распространение радио/телепрограмм эротического характера без кодирования – только с 23:00 до 04:00. Розничная продажа – в запечатанных прозрачных упаковках в спецместах. **V. Отношения с гражданами и организациями – Права и обязанности редакции:** * **Запрос информации (Ст. 39, 40):** Редакция имеет право запрашивать информацию. Отказ возможен, если сведения – гостайна, коммерческая или иная охраняемая тайна (уведомление в 3-дневный срок). Отсрочка – если не могут представить в 7-дневный срок (уведомление в 3-дневный срок). * **Конфиденциальность информации (Ст. 41) – КРИТИЧЕСКИ ВАЖНО:** * Не разглашать сведения, предоставленные гражданином с условием сохранения их в тайне. * **Сохранять в тайне источник информации** и не называть лицо, предоставившее сведения анонимно, _кроме как по требованию суда_ по находящемуся в его производстве делу. * Особые правила по информации о несовершеннолетних (см. Ст. 4). * **Авторские произведения и письма (Ст. 42):** Соблюдать авторские права. Письма в редакцию могут быть использованы, если не искажен смысл и не нарушен закон. Редакция не обязана отвечать на письма. Никто не может обязать редакцию опубликовать отклоненный материал (если иное не предусмотрено законом). * **Право на опровержение (Ст. 43, 44):** Если распространены не соответствующие действительности и порочащие честь/достоинство сведения, гражданин/организация вправе требовать опровержения. Редакция обязана опровергнуть, если не докажет обратное. * **Порядок:** тот же шрифт, то же место (под заголовком "Опровержение"). По радио/ТВ – в то же время, в той же передаче. Объем – не более чем вдвое больше опровергаемого. Сроки: 10 дней (для еженедельников и чаще), в ближайшем выпуске (для остальных). Уведомить о сроке или отказе – в течение месяца. * **Основания отказа в опровержении (Ст. 45):** Злоупотребление свободой СМИ, противоречие решению суда, анонимность. Может быть отказано, если уже опровергнуто или прошел 1 год со дня распространения. * **Право на ответ (Ст. 46):** Если сведения не соответствуют действительности или ущемляют права/интересы. Порядок аналогичен опровержению. **VI. Права и обязанности журналиста (важно для редактора, т.к. редакция несет ответственность):** * **Права журналиста (Ст. 47):** Искать, запрашивать, получать, распространять информацию; посещать госорганы/организации; быть принятым должностными лицами; доступ к документам (кроме тайн); копировать, публиковать (с соблюдением авторских прав); производить записи (аудио, видео, фото), кроме запрещенных законом случаев; посещать места ЧС, митинги; проверять информацию; излагать личные суждения; отказаться от подготовки материала, противоречащего убеждениям; снять подпись; распространять под псевдонимом или анонимно. * **Выполнение поручения в особых условиях (Ст. 47.1):** Редакция обязана обеспечить обучение безопасности, средства защиты, содействовать в получении удостоверения (для выезда за рубеж). Предусмотрены компенсации при вреде жизни/здоровью. * **Аккредитация (Ст. 48):** Право редакции подать заявку. Органы обязаны аккредитовать при соблюдении правил. Лишение аккредитации – за нарушение правил или распространение порочащих сведений, подтвержденное судом, или по Ст. 56.2 (если СМИ лишено регистрации). * **Обязанности журналиста (Ст. 49) – РЕДАКТОР КОНТРОЛИРУЕТ:** * Соблюдать устав редакции. * **Проверять достоверность сообщаемой информации.** * Удовлетворять просьбы об указании источника или авторизации цитаты. * Сохранять конфиденциальность информации/источника. * Получать согласие на распространение сведений о личной жизни (кроме защиты общественных интересов). * При получении информации ставить в известность о проведении аудио/видео/фотосъемки. * Ставить в известность главреда о возможных исках. * Отказаться от задания, если оно незаконно. * Предъявлять удостоверение. * Соблюдать запрет на предвыборную агитацию при проф. деятельности. * Уважать права, честь, достоинство граждан и организаций. * **Скрытая запись (Ст. 50):** Допускается, если не нарушает конституционные права, необходима для защиты общественных интересов (с мерами против идентификации посторонних), или по решению суда. * **Недопустимость злоупотребления правами журналиста (Ст. 51):** Нельзя использовать права для сокрытия/фальсификации, распространения слухов, сбора информации для посторонних, для дискредитации по признакам пола, возраста, расы, религии и т.д. **VII. Ответственность и освобождение от нее:** * **Ответственность (Ст. 56, 59, 60):** Учредители, редакции, издатели, распространители, журналисты и др. несут ответственность за нарушения. * **Освобождение от ответственности (Ст. 57) – ВАШИ "ЩИТЫ":** Редакция, главред, журналист НЕ несут ответственности за распространение недостоверных/порочащих сведений, если они: 1. Присутствуют в обязательных сообщениях (решения суда, сообщения регистрирующего органа). 2. Получены от информационных агентств. 3. Содержатся в ответе на запрос информации или в материалах пресс-служб госорганов, организаций. 4. Являются дословным воспроизведением выступлений депутатов, официальных лиц. 5. Содержатся в авторских произведениях, идущих в эфир без предварительной записи, или в текстах, не подлежащих редактированию по закону. 6. Являются дословным воспроизведением сообщений другого СМИ (которое может быть установлено и привлечено к ответственности), _кроме информации, указанной в ч.6 ст.4 (о несовершеннолетних-жертвах) и п.1-6 ч.1 ст.56.2 (фейки, дискредитация и т.д.)._ * **Внимание:** Воспроизведение агитматериалов другого СМИ в период выборов не освобождает от ответственности, если нарушены правила агитации. * **Возмещение морального вреда (Ст. 62):** По решению суда, если распространены не соответствующие действительности сведения, порочащие честь и достоинство. **VIII. Тонкости и недавние изменения:** * **Маркировка "иностранный агент":** Обязательна для материалов самого иноагента и при упоминании его. Запрет рекламы ресурсов иноагентов. * **Статья 56.2 о приостановлении/прекращении деятельности по требованию Генпрокурора:** Очень серьезный инструмент. Подразумевает быструю реакцию Роскомнадзора. Расширяет список "запрещенного" контента, за который могут быть немедленные санкции. * **Ответственность за перепечатку (Ст. 57 п. 6):** "Щит" не работает для перепечатки информации о несовершеннолетних-жертвах и информации, указанной в новой статье 56.2 (фейки, дискредитация ВС РФ и т.д.). То есть, перепечатывая такой контент, вы берете ответственность на себя. * **Информация о несовершеннолетних (Ст. 4, 41):** Очень строгие правила. Получение согласия – ключевой момент, кроме особых случаев. **Рекомендации редактору:** 1. **Знайте свои права и обязанности.** 2. **Тщательно проверяйте информацию перед публикацией.** 3. **Обучайте журналистов нормам закона и этики.** 4. **Особое внимание – к темам, перечисленным в Ст. 4 и Ст. 56.2.** 5. **Правильно оформляйте выходные данные и маркировку.** 6. **Храните архивы, особенно предвыборные материалы.** 7. **Консультируйтесь с юристом по сложным вопросам.** 8. **Будьте в курсе изменений законодательства – они происходят часто.** Этот документ не заменяет полного текста закона, но должен помочь в повседневной работе. Удачи!
"""

@mcp.tool()
async def check_news_law(news_text: str, ctx: Context) -> str:
    """
    Принимает текст новости, анализирует его на соответствие законодательству
    с использованием OpenAI API и специального промпта, и возвращает результат анализа.
    """
    ctx.info(f"Инструмент 'check_news_law' получил новость для анализа: {news_text[:70]}...")
    
    try:
        analysis_result = await call_openai_for_news_analysis(
            news_text=news_text,
            analysis_prompt=NEWS_ANALYSIS_LEGAL_PROMPT,
            ctx=ctx
        )
        return analysis_result
    except ValueError as ve: # Ловим ошибку отсутствия API ключа
        return str(ve)
    except Exception as e:
        ctx.error(f"Неожиданная ошибка в инструменте check_news_law: {str(e)}")
        return f"Внутренняя ошибка сервера при анализе новости. {str(e)}"


if __name__ == "__main__":
    mcp.run()
